from src.api import MWApi
from bs4 import BeautifulSoup, element
from src.data import forbidden_tags, new_line_tags


class MWParser:
    def __init__(self, cache_folder, URL="https://en.wikipedia.org/w/api.php"):
        self.api = MWApi(URL, cache_folder)
        self.parsed = None

    def parse_wikicode(self, text):
        soup = BeautifulSoup(self.api.parse(text), "html.parser")
        root = soup.find("div", {"class": "mw-parser-output"})
        self.parsed = ParserJob(root)
        return self.parsed

    def parse_html(self, html):
        soup = BeautifulSoup(html, "html.parser")
        root = soup.find()
        self.parsed = ParserJob(root)
        return self.parsed

    def get_text_from_tag(self):
        pass


class ParserJob:
    def __init__(self, root):
        self.root = root
        self.preformatted = False
        self.text = [""]
        self.data = []
        self.textLocation = 0
        self.process(root)
        self.repair_text()

    def repair_text(self):
        if not self.text[-1]:
            self.text.pop()
        self.text = [line.strip() for line in self.text]

    def process(self, section):
        {
            element.Tag: self.process_tag,
            element.NavigableString: self.process_string,
            element.Comment: lambda a: None,
        }[type(section)](section)

    def process_tag(self, tag):

        for forbidden_tag in forbidden_tags:
            if forbidden_tag["name"] == tag.name:
                match = True
                for name, value in forbidden_tag["attrs"].items():
                    if not (name in tag.attrs and tag.attrs[name] == value):
                        match = False
                if match:
                    return

        if tag.name in new_line_tags:
            if len(self.text[self.textLocation]) > 0:
                self.textLocation += 1
                self.text.append("")
        obj = {
            "tag": tag.name,
            "attrs": tag.attrs,
            "start": [self.textLocation, len(self.text[self.textLocation])],
        }
        if tag.name == "pre":
            self.preformatted = True

        for child in tag.children:
            self.process(child)

        if tag.name == "pre":
            self.preformatted = False

        obj["end"] = [self.textLocation, len(self.text[self.textLocation]) - 1]

        if tag.name in new_line_tags:
            if len(self.text[self.textLocation]) > 0:
                self.textLocation += 1
                self.text.append("")
        self.data.append(obj)

    def process_string(self, string):
        if self.preformatted:
            self.process_preformatted_string(string)
            return
        if string == "\n":
            return
        fixed_str = self.fix_string(string)
        self.text[self.textLocation] += fixed_str

    def process_preformatted_string(self, string):
        self.text[self.textLocation] += string

    def fix_string(self, string):
        string = string.replace("\r\n", " ")
        string = string.replace("\n", " ")
        while "  " in string:
            string = string.replace("  ", " ")
        if (
            self.text[self.textLocation]
            and self.text[self.textLocation][-1] == " "
            and string[0] == " "
        ):
            string = string[1:]
        return string

    def find_first_tag(self, expected_tag):
        for tag in self.data:
            if tag["tag"] == expected_tag:
                return tag
        return {"tag": None}



import requests
import hashlib
from os.path import *


class MWApi:
    def __init__(self, URL, cache_folder):
        self.URL = URL
        self.cache_folder = cache_folder

    def parse(self, text):
        cached = self.get_cached(text)
        if cached:
            return cached
        response = requests.get(
            url=self.URL,
            params={
                "action": "parse",
                "contentmodel": "wikitext",
                "text": text,
                "format": "json",
            },
        )
        parsed = response.json()["parse"]["text"]["*"]
        self.save_to_cache(text, parsed)
        return parsed

    def get_cached(self, text):
        name = hashlib.sha256(text.encode("utf-8")).hexdigest()
        path = f"{self.cache_folder}/{name}"
        if exists(path):
            with open(path, "r", encoding="utf-8") as file:
                parsed = file.read()
                return parsed

    def save_to_cache(self, text, parsed):
        name = hashlib.sha256(text.encode("utf-8")).hexdigest()
        path = f"{self.cache_folder}/{name}"
        with open(path, "w", encoding="utf-8") as file:
            # print(parsed)
            file.write(parsed)



from setuptools import setup, find_packages


setup(
    name='MMK_parser',
    version='0.1.6',
    license='MIT',
    author="MMK_group",
    author_email='maciekgoncerzewicz@gmail.com',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    url='https://github.com/gmyrianthous/example-publish-pypi',
    keywords='example project',
    install_requires=[
        'bs4',
        'requests',
    ],
)


forbidden_tags = [
    {"name": "style", "attrs": {}},
    {"name": "script", "attrs": {}},
    {"name": "span", "attrs": {"class": ["IPA"]}},
    {"name": "span", "attrs": {"class": ["unicode haudio"]}},
    {"name": "span", "attrs": {"class": ["mw-editsection"]}},
    {"name": "table", "attrs": {"class": ["infobox vcard"]}},
    {"name": "sup", "attrs": {"class": ["reference"]}},
    {"name": "div", "attrs": {"role": "navigation"}},
    {"name": "table", "attrs": {}},
    {"name": "span", "attrs": {"class": ["error", "mw-ext-cite-error"]}},
    {"name": "sup", "attrs": {"class": ["reference"]}},
]

new_line_tags = ["p", "br", "hr", "li", "ol", "dl", "dt", "dd"]



import json
from MWParser import MWParser

f = open("test.txt", "a")


class Iterator:
    def __init__(self, file_name):
        self.html_data = open(file_name, "rb")
        self.parser = MWParser("./cached")

    def __iter__(self):
        return self

    def __next__(self):
        line = self.html_data.readline()
        json_data = json.loads(line)
        return self.parser.parse_html(json_data["article_body"]["html"])


for html in Iterator("../simplewiki-NS0-20220420-ENTERPRISE-HTML.json/simplewiki_0.ndjson"):
    print(html.text)
